VERSION 5.00
Begin VB.UserControl ipUC 
   ClientHeight    =   3600
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   4800
   ScaleHeight     =   3600
   ScaleWidth      =   4800
   Begin VB.Timer Timer 
      Enabled         =   0   'False
      Interval        =   5000
      Left            =   1320
      Top             =   2640
   End
End
Attribute VB_Name = "ipUC"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'-- Getting IP -> Host name asynchronous. The return from WSAAsyncGetHostByAddr is
'-- sent as a windows message to a designated hWnd. The trick of using a Textbox_MouseUp
'-- event sub won't work because the task handle -- the numeric ID returned when
'-- WSAAsyncGetHostByAddr is called -- is returned in the wParam parameter. The MouseUp
'   event doesn't get the wParam value by the time the form has processed the message.
'-- The task handle is needed to know whether a return message matches the current call
'   or another call that has already timed out.
' So.... the parent form hWnd (F2.hWnd) is sent to WSAAsyncGetHostByAddr and the form
' is subclassed. When the subclass gets the message, it raises an event here, WSAReturn,
' sending it the wParam and lParam values. That works.
'
'  The general operation, then, is like so: GetHostNameA is called, passing an IP address.
' That calls WSAAsyncGetHostByAddr (if winsock has been started, IP checks out, etc.)
'  If return from that call is non-zero it's saved in HASync variable as the task handle,
'  and the timer is started. If timeout elapses before the call returns then HASync is
' set to 0 and the event IPResolved is fired, returning "" as the host name. If a
' return comes through the subclass before the timer runs out, and the wParam value
' matches HASync, and the lParam value indicates no errors, then the host name is
' retrieved and sent back in the IPResolved event.
'  The class IPInfo is "fronting" all of this, presenting a neater interface for
' doing the operation and providing a Busy (boolean) property to allow VBScripts
' to function while waiting for the return event to fire.
'  Using the class, one calls EnableHostNameData, which starts winsock and returns
' 0 on success. Then one can call GetHostNameFromIP as many times as desired. Each
' call will result in an OnHostNameResult event (which should happen within the timeout
' span). OnHostNameResult returns the IP address that was sent in, along with its
' matching host name, or "" if host name is not available. On failure it also returns
' an error code.

Private Const MAXGETHOSTSTRUCT = 2048  ' supposed to be minimum 1024, but let's be generous
Private Const WM_LBUTTONUP = &H202       ' because this buffer has to store not only the
                                        ' HOSTENT structure but also its contents.
'Public Type HOSTENT
'    hName     As Long
'    hAliases  As Long
'    hAddrType As Integer
'    hLength   As Integer
'    hAddrList As Long
'End Type

Private Declare Function WSAAsyncGetHostByAddr Lib "ws2_32.dll" (ByVal hwnd As Long, ByVal wMsg As Long, ByRef lngAddr As Long, ByVal lngLenght As Long, ByVal lngType As Long, buf As Any, ByVal lngBufLen As Long) As Long

'------------------------
'-- According to MS this function does not work and can cause erratic behavior!!!
'-- The official recommendation to cancel a call to WSAAsyncGetHostByAddr is to ignore
'-- the return. So that's what this UC does.
'Private Declare Function WSACancelAsyncRequest Lib "ws2_32.dll" (ByVal taskhandle As Long) As Long
 '-----------------------
 
Private HTimer As Long  '-- timer handle for SetTimer
Private HASync As Long  '-- handle for async call.
Private SockOK As Boolean  '-- this boolean is separately in both this UC and in the IPInfo class.
Private sCurIP As String '-- store current IP being translated.
Private BufHOSTENT() As Byte

  '-- This event provides a way to return the result of asynchronous
  '-- IP->HostName request.
Public Event IPResolved(sIP As String, sHost As String, LErr As Long)
   
    '-- access provided here for public property in class that sets timeout.
Public Property Let TimeOutUC(iTime As Integer)
     On Error Resume Next
   Timer.Interval = iTime
End Property

 '-- called with IPInfo class-initialize to start winsock.
Public Function StartUp() As Long
  Dim LRet As Long
  Dim WData As WSAData  'structure to pass to WSAStartup as an argument
     On Error Resume Next
  
   LRet = WSAStartup(&H101, WData)
      If (LRet <> 0) Then
          Select Case LRet
             Case WSASYSNOTREADY
               '  iErrCode = 101  ' "System not ready for network communication."
                 StartUp = CVar(1)  ' system not ready for network communication.
              Case WSAVERNOTSUPPORTED
                ' iErrCode = 102  '"Winsock version 1.1 not supported."
                 StartUp = CVar(2)  ' version 1.1 not supported.
              Case WSAEINPROGRESS
               '  iErrCode = 103  ' "A blocking winsock is already in use."
                 StartUp = CVar(3)  ' a blocking winsock already in use.
              Case WSAEPROCLIM
               '  iErrCode = 104   ' "The limit on number of winsock tasks has been reached."
                 StartUp = CVar(4) ' limit on number of winsock tasks has been reached.
              Case WSAEFAULT
               '  iErrCode = 105 '  "Error with parameter of WSAStartup call."
                 StartUp = CVar(5)  '  WData is not a valid pointer.
              Case Else
                 'iErrCode = 115  ' Unknown error.
                 StartUp = CVar(6)
         End Select
      Else
        SockOK = True
        StartUp = CVar(0)
      End If
End Function

     '-- make asynchronous call to get host name from IP.
     '-- data is returned with IPResolved event.
 Public Sub GetHostNameA(sIP As String)
  Dim sBuf As String
  Dim LIP As Long
        On Error Resume Next
   sCurIP = sIP
     If (SockOK = False) Then
       RaiseEvent IPResolved(sCurIP, "", 1)  '-- error 1: no winsock initialized.
       HASync = 0
       Exit Sub
     End If
    LIP = inet_addr(sIP)
   If (LIP = INADDR_NONE) Then
      RaiseEvent IPResolved(sCurIP, "", 2) '-- error 2: not a valid IP address.
      HASync = 0
      Exit Sub
   End If   '-- invalid IP string.
   ReDim BufHOSTENT(2047) As Byte
   HASync = WSAAsyncGetHostByAddr(F2.hwnd, WM_LBUTTONUP, LIP, 4, AF_INET, BufHOSTENT(0), MAXGETHOSTSTRUCT)
     If (HASync = 0) Then
        RaiseEvent IPResolved(sCurIP, "", 3) '-- Error 3: WSAAsyncGetHostByAddr failed.
        Exit Sub
     End If
  '-- if ok at this point then call timer with timeout interval.
    Timer.Enabled = True
     
 End Sub
 
 '-- wParam doesn't come through using textbox_MouseUp. The parent form
 '-- is apparently processing that value out. So the parent form, F2, is
 '-- being subclassed instead. Subclass hooks on IPInfo initialize, when it
 '-- loads the form, then unhooks on IPInfo terminate, when it unloads
 '-- the form.
 '-- wParam should contain HAsync value, the task handle from call to
 '-- WSAAsyncGetHostByAddr. HiWord of lparam will contain any error code.
 
 '-- The idea here is to call WSAAsyncGetHostByAddr, store the task handle
 '-- returned, then wait up to TimeOut seconds. If the hostname comes back
 '-- before the timeout elapses then wParam here will match HASync, the
 '-- task handle value returned by the function call. If no match then skip it,
 '-- as that will be a case of timed out return.
 '-- After checking that it's a return for current call, also check
 '-- that the hiword of lParam is 0, indicating no error.
 
Public Sub WSAReturn(wParam As Long, lParam As Long)
   Dim tHost As HOSTENT
   Dim sBuf As String
    '-- if this is the return for THIS call, and not a return for a former call that's
    '-- already timed out then wParam = HASync. If not then don't do anything because
    '-- a non=match means that returning call has already timed out.
     If (wParam = HASync) Then
      If GetHiWord(lParam) = 0 Then
        CopyMemory tHost, BufHOSTENT(0), LenB(tHost)
        sBuf = String$(256, 0)
        CopyMemory2 ByVal sBuf, ByVal tHost.hName, 256
        RaiseEvent IPResolved(sCurIP, Left$(sBuf, InStr(1, sBuf, Chr(0)) - 1), 0)
      Else
        RaiseEvent IPResolved(sCurIP, "", 4) '-- Error 4: Probably either no host found or
                                             '-- no host name to return for IP.
      End If
        Timer.Enabled = False
        HASync = 0
     End If
End Sub

Private Function GetHiWord(ByVal LongIn As Long) As Integer
  On Error Resume Next
   GetHiWord = (LongIn And &HFFFF0000) \ &H10000
End Function

  '-- called with ipinfo class-terminate to close winsock.
Public Sub CloseDown()
  Dim LRet As Long
   On Error Resume Next
    If (SockOK = True) Then
      LRet = WSACleanup()
      SockOK = False
    End If
      Timer.Enabled = False
End Sub

  '-- if timeout elapses before call returns then cancel the call and return value in event.
Private Sub Timer_Timer()
  Dim LRet As Long
    On Error Resume Next
  If (HASync <> 0) Then
    RaiseEvent IPResolved(sCurIP, "", 5) '-- Error 5: Timed out.
    HASync = 0
  End If
    Timer.Enabled = False
End Sub
